#include "gpsroutines.h"
#include "timeroutines.h"
#include "strings.h"
#include "math.h"

#define NMEA_Cmds_Len 5
#define TIMEZONE_OFFSET 3

char  * NMEA_Cmds[] = { "$GPGGA", "$GPGSV", "$GPTXT", "$GPGSA", "$GPRMC" };


//   
uint8_t NMEA_Get_Cmd_Code(char * str)
{
	uint8_t i;
	char * curr_cmd;

	for (i = 1; i <= NMEA_Cmds_Len; i++)
	{
		curr_cmd = NMEA_Cmds[i-1];

		if (STR_StartFrom(str, curr_cmd))
			return i;
	}
	return 0;
}


//     04831.4406
GPS_COORD coord_str_to_packed(char * string)
{
	GPS_COORD res;
	uint8_t tmp;
	uint16_t dp_minutes;

	// 
	tmp = 100 * (*string++ - '0');
	tmp += 10 * (*string++ - '0');
	tmp += (*string++ - '0');

	//    hi  
	res.degrees = tmp;

	// 
	tmp = 10 * (*string++ - '0');
	tmp += (*string++ - '0');

	//    hi  
	res.minutes = tmp;

	string++; //    .

	//     
	dp_minutes = 1000 * (*string++ - '0');
	dp_minutes += 100 * (*string++ - '0');
	dp_minutes += 10 * (*string++ - '0');

	dp_minutes += *string++ - '0';

	res.dp_minutes = dp_minutes;

	return res;
}

//        
// $GPRMC,100504.000,A,4831.4385,N,03456.8624,E,3.9,051.1,280611,,,A*65
//      ,   
//  rtc_set = 1    
GPS_GPRMC NMEA_Parse_GPRMC(char * string)
{
	GPS_GPRMC res;
	string += 7;
	int year, month, day, hour, min, sec;

        res.CTIME[0] = (uint8_t) (*string - '0');
        hour = (*string++ - '0') * 10;

        res.CTIME[1] = (uint8_t) (*string - '0');
	hour += (*string++ - '0');

        res.CTIME[1] += TIMEZONE_OFFSET;
        if (res.CTIME[1] > 9)
        {
          res.CTIME[1] -= 10;
          res.CTIME[0]++;
        }
        if ((res.CTIME[0] == 2) && (res.CTIME[1] > 3))
          {
            res.CTIME[0] = 0;
            res.CTIME[1] -= 4;
          }
        res.CTIME[2] = (uint8_t) (*string - '0');
	min = (*string++ - '0') * 10;

        res.CTIME[3] = (uint8_t) (*string - '0');
	min += (*string++ - '0');
	sec = (*string++ - '0') * 10;
	sec += (*string - '0');

	string += 6; //   ,   

	res.VALID = GPS_GPRMC_INVALID;

	if (*string++ == 'A')
        {
          res.VALID =  GPS_GPRMC_VALID; // valid data

          *string = '0';
          res.LATITUDE = coord_str_to_packed(string);

          string += 11;

          if (*string == 'N')
            res.LATITUDE.degrees |= GPS_GPRMC_NORTH_LAT; //  


          string += 2;

          res.LONGITUDE = coord_str_to_packed(string);

          string += 11;

          if (*string == 'E')
            res.LONGITUDE.degrees |= GPS_GPRMC_EAST_LONG; //  

          string += 1;

          //     speed in knots

          char * string1 = string;

          string++;

          while (*string != ',') string++; //    .

          char * string2 = string; //   

          string--; //      
          uint16_t multiplier = 1;
          res.GPS_SPEED = 0;

          while (string != string1)
          {
	    if (*string != '.')
		{
			res.GPS_SPEED += (*string - '0') * multiplier;
			multiplier *= 10;
		}
		string--;
          }

          uint32_t sp_kmh = (uint32_t) res.GPS_SPEED;
          sp_kmh  *= 1852;
          sp_kmh = sp_kmh / 10000;
          res.GPS_SPEED = (uint16_t) sp_kmh;

          res.KSPEED[0] = res.GPS_SPEED / 1000;
          uint16_t sp_kmh1 = res.GPS_SPEED - res.KSPEED[0] * 1000;
          res.KSPEED[1] = sp_kmh1 / 100;
          sp_kmh1 = sp_kmh1 - res.KSPEED[1] * 100;
          res.KSPEED[2] = sp_kmh1 / 10;
          sp_kmh1 = sp_kmh1 - res.KSPEED[2] * 10;
          res.KSPEED[3] = sp_kmh1;

          uint8_t i;
          for (i = 0; i < 4; i++)
            if (res.KSPEED[i] == 0)
              res.KSPEED[i] = 0x10;
          //  ,      .
          string = string2 + 7; //     heading
          day = (*string++ - '0') * 10;
          day += (*string++ - '0');
          month = (*string++ - '0') * 10;
          month += (*string++ - '0');
          year = (*string++ - '0') * 10;
          year += (*string++ - '0');
          year += 2000;

          //calc_unix_time(year, month, day, hour, min, sec, &res.TIME);
        }
   // ,   .      .
   return res;
}

/*
double Calc_Distance(double latitude1, double longitude1, double latitude2, double longitude2)
{
	double a, b, s, d, m, bs, ad;

	m = (latitude2 + latitude1) / 2;

	if (m < 50)
	{
		a = 111.33 - 0.0156 * m * m - 0.023 * m;
	}
	else
	{
		a = 135.35 - 0.00586 * m * m - 0.978 * m;
	}

	b = 110.44 + 0.014 * m;

	s = latitude1 - latitude2;
	d = longitude1 - longitude2;

	bs = b * s;
	bs = bs * bs;

	ad = a * d;
	ad = ad * ad;

	m = (bs + ad);

	m = sqrt(m);

	return m;

}
*/

double haversine_km(double lat1, double long1,
                    double lat2, double long2)
{
    #define d2r (3.141592 / 180.0)
    #define sindif(v1, v2) sin(((v1 - v2) * d2r) / 2.0)

    double sdlong = sindif(long2, long1);
    double sdlat  = sindif(lat2, lat1);
    lat1 *= d2r;
    lat2 *= d2r;
    sdlong *= sdlong;
    sdlat  *= sdlat;
    double a = sdlat + cos(lat1) * cos(lat2) * sdlong;

    double res = 6367 * 2 * atan2(sqrt(a), sqrt(1 - a));
    return res;
}