#include "stm8s.h"
#include "stm8s_tim2.h"
#include "stm8s_tim4.h"
#include "stm8s_gpio.h"
#include "stm8s_clk.h"
#include "stm8s_itc.h"
#include "stm8s_uart1.h"

#include "gpsroutines.h"


uint32_t sys_msec = 0;
uint32_t sys_sec = 0;

uint16_t step = 1;
uint16_t pwm = 1;
uint16_t timeout = 0;

#define UART_RX_BUF_LEN 90
#define UART_RX_BUF_LN_MAX 2
uint8_t UART_RX_BUF[2][90];
uint8_t UART_RX_POS = 0;
uint8_t UART_RX_LN = 0;
uint8_t UART_PREV_SYM = 0;
uint8_t UART_CURR_SYM = 0;
uint8_t UART_ACTUAL_LN = 10;

//      
uint8_t LED_SYM_OFF[3] = {0xFB, 0x07, 0xCF}; //   .
uint8_t LED_SYM[][3] =
{
  //       PORTD_DDR, PORTC_DDR, PORTB_DDR
  {0x04, 0xF8, 0x00}, //0
  {0x00, 0xC0, 0x00}, //1
  {0x04, 0xB0, 0x20}, //2
  {0x04, 0xE0, 0x20}, //3
  {0x00, 0xC8, 0x20}, //4
  {0x04, 0x68, 0x20}, //5
  {0x04, 0x78, 0x20}, //6
  {0x04, 0xC0, 0x00}, //7
  {0x04, 0xF8, 0x20}, //8
  {0x04, 0xE8, 0x20}, //9
  {0x00, 0x00, 0x00}, //A
  {0x00, 0x00, 0x00}, //B
  {0x00, 0x00, 0x00}, //C
  {0x00, 0x00, 0x00}, //D
  {0x00, 0x00, 0x00}, //E
  {0x00, 0x00, 0x00}, //F
  {0x00, 0x00, 0x00}, //10 
  {0x00, 0x00, 0x00}, //11
  {0x00, 0x00, 0x00}, //12
  {0x00, 0x00, 0x00}, //13
};

uint8_t LED_BUF[4] = {0x10, 0x10, 0x10, 0x10}; //    
uint8_t LED_BUF_DP[4] = {0x00, 0x00, 0x00, 0x00}; //    

#define LED_OFF_MASK1 0x12 //PD4 + PD1
#define LED_OFF_MASK2 0x06 // PA1 + PA2
uint8_t LED_ON_MASK[][2] =
{
  {0xFD, 0xFF},
  {0xFF, 0xFB},
  {0xFF, 0xFD},
  {0xEF, 0xFF}
};
#define LED_SYM_DISPLAY_MAX 4  //     
uint8_t LED_SYM_DISPLAY = 0; //   


void LED_OUT_SYM(uint8_t sym_code);

//   4
INTERRUPT_HANDLER(TIM4_handler,ITC_IRQ_TIM4_OVF)
{
  TIM4->SR1 = (uint8_t)(~((uint8_t)(TIM4_IT_UPDATE)));
  sys_msec++;
  if (sys_msec == 1000)
  {
    sys_sec++;
    sys_msec = 0;
  }
  //  

  //   .     1,     
  //  .
  GPIOD->ODR |= LED_OFF_MASK1;
  GPIOA->ODR |= LED_OFF_MASK2;
  LED_OUT_SYM(LED_BUF[LED_SYM_DISPLAY]);
  if (LED_BUF_DP[LED_SYM_DISPLAY])
  {
    //   
    GPIOB->DDR |= 0x10; //  PB4  
  }
  GPIOD->ODR &= LED_ON_MASK[LED_SYM_DISPLAY][0];
  GPIOA->ODR &= LED_ON_MASK[LED_SYM_DISPLAY][1];
  LED_SYM_DISPLAY++;
  if (LED_SYM_DISPLAY ==  LED_SYM_DISPLAY_MAX)
    LED_SYM_DISPLAY = 0;
}

//        
INTERRUPT_HANDLER(UART1_RX, ITC_IRQ_UART1_RX)
{
    UART1->SR = (uint8_t)~(UART1_SR_RXNE);
    UART_CURR_SYM =  ((uint8_t)UART1->DR);

    if ((UART_CURR_SYM == 0x0A) && (UART_PREV_SYM == 0x0D))
    {
      //UART_PREV_SYM = UART_CURR_SYM;
      UART_RX_POS = 0;
      UART_ACTUAL_LN = UART_RX_LN;
      UART_RX_LN++;
      if (UART_RX_LN == UART_RX_BUF_LN_MAX)
        UART_RX_LN = 0;
      return;
    }

    if (UART_RX_POS == UART_RX_BUF_LEN)
      UART_RX_POS = 0; // ,  
    UART_RX_BUF[UART_RX_LN][UART_RX_POS++] = UART_CURR_SYM ;
    UART_PREV_SYM = UART_CURR_SYM;
}


void LED_OUT_SYM(uint8_t sym_code)
{
  GPIOD->DDR = (GPIOD->DDR & LED_SYM_OFF[0]) | LED_SYM[sym_code][0];
  GPIOC->DDR = (GPIOC->DDR & LED_SYM_OFF[1]) | LED_SYM[sym_code][1];
  GPIOB->DDR = (GPIOB->DDR & LED_SYM_OFF[2]) | LED_SYM[sym_code][2];
}

void init_hardware(void)
{
  CLK_DeInit();
  CLK_SYSCLKConfig(CLK_PRESCALER_HSIDIV1);
  CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSI, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);;

  //  .
  GPIO_Init(GPIOD,GPIO_PIN_3,GPIO_MODE_OUT_PP_LOW_FAST);
  //  2   PWM 2   1
  TIM2_TimeBaseInit(TIM2_PRESCALER_32, 999);

  //      
  TIM2_OC2Init(TIM2_OCMODE_PWM2, TIM2_OUTPUTSTATE_ENABLE, 1, TIM2_OCPOLARITY_LOW);
  TIM2_Cmd(ENABLE);

  //  
   CLK_PeripheralClockConfig(CLK_PERIPHERAL_TIMER4,ENABLE);
  TIM4_TimeBaseInit(TIM4_PRESCALER_128, 125 - 1);
  TIM4_ClearFlag(TIM4_FLAG_UPDATE);
  TIM4_UpdateRequestConfig(TIM4_UPDATESOURCE_REGULAR);
  TIM4_ITConfig(TIM4_IT_UPDATE,ENABLE);
  TIM4_Cmd(ENABLE);

  //      GPS 
  GPIO_Init(GPIOD,GPIO_PIN_5,GPIO_MODE_OUT_PP_HIGH_FAST);
  GPIO_Init(GPIOD,GPIO_PIN_6,GPIO_MODE_IN_FL_NO_IT);


  UART1_DeInit();
  UART1_Init(9600, UART1_WORDLENGTH_8D, UART1_STOPBITS_1, UART1_PARITY_NO,
             UART1_SYNCMODE_CLOCK_DISABLE, UART1_MODE_TXRX_ENABLE);
  UART1_ITConfig(UART1_IT_RXNE, ENABLE);
  UART1_Cmd(ENABLE);

  // 
  GPIO_Init(GPIOA,GPIO_PIN_3,GPIO_MODE_IN_PU_NO_IT);

  //  ,   .
  //  "" ,   ,   
  // ,    -      - .
  //   ,   1,7      
  // 
  GPIO_Init(GPIOD,GPIO_PIN_2,GPIO_MODE_IN_FL_NO_IT);
  //GPIO_Init(GPIOD,GPIO_PIN_3,GPIO_MODE_IN_FL_NO_IT);
  GPIO_Init(GPIOC,GPIO_PIN_7,GPIO_MODE_IN_FL_NO_IT);
  GPIO_Init(GPIOC,GPIO_PIN_6,GPIO_MODE_IN_FL_NO_IT);
  GPIO_Init(GPIOC,GPIO_PIN_5,GPIO_MODE_IN_FL_NO_IT);
  GPIO_Init(GPIOC,GPIO_PIN_4,GPIO_MODE_IN_FL_NO_IT);
  GPIO_Init(GPIOB,GPIO_PIN_4,GPIO_MODE_IN_FL_NO_IT);
  GPIO_Init(GPIOB,GPIO_PIN_5,GPIO_MODE_IN_FL_NO_IT);
  GPIOD->ODR &= 0xF7;
  GPIOC->ODR &= 0x07;
  GPIOB->ODR &= 0xCF;

  GPIOD->CR2 = 0;
  GPIOC->CR2 = 0;
  GPIOB->CR2 = 0;
  GPIOA->CR2 = 0;

  //  
  GPIO_Init(GPIOD,GPIO_PIN_1,GPIO_MODE_OUT_PP_HIGH_SLOW);
  GPIO_Init(GPIOD,GPIO_PIN_4,GPIO_MODE_OUT_PP_HIGH_SLOW);
  GPIO_Init(GPIOA,GPIO_PIN_1,GPIO_MODE_OUT_PP_HIGH_SLOW);
  GPIO_Init(GPIOA,GPIO_PIN_2,GPIO_MODE_OUT_PP_HIGH_SLOW);

}

#define SPEED_TABLE_DIV 28
#define SPEED_TABLE_MAX 130
#define SPEED_OVER_PWM 600
#define SPEED_OVER_VALUE 183

uint16_t speed_table[] = {0, 1, 10, 150, 20, 190, 30, 220, 40, 250, 50, 280, 60, 300, 70, 330, 80, 350, 90, 370, 100, 385, 110, 435, 120, 480, 130, 520 };

uint16_t calc_pwm_speed(uint16_t speed)
{
  uint32_t speed1, speed2, pwm1, pwm2;

  if (speed > SPEED_TABLE_MAX)
  {
    speed1 = speed_table[SPEED_TABLE_DIV - 2];
    pwm1 = speed_table[SPEED_TABLE_DIV - 1];
    speed2 = SPEED_OVER_VALUE;
    pwm2 = SPEED_OVER_PWM;
  }
  else
  {
    uint8_t i;
    for (i = 0; i < SPEED_TABLE_DIV; i = i + 2)
    {
      if (speed_table[i] >= speed)
      {
        speed2 = speed_table[i];
        pwm2 = speed_table[i + 1];
        if (i < 2)
        {
          speed1 = 0; pwm1 = 1;
        }
        else
        {
          speed1 = speed_table[i-2]; pwm1 = speed_table[i-1];
        }
        break;
      }
    }
  }
  //  , ,  .
  speed2 = speed2 - speed1; pwm2 = pwm2 - pwm1;
  pwm2  = pwm2 * 1000;
  pwm2 = pwm2 / speed2;
  speed1 = ((uint32_t) speed) - speed1;
  pwm2 = pwm2 * speed1;
  pwm1 = pwm1 + (pwm2 / 1000);
  return (uint16_t) pwm1;
}

int main( void )
{

  //Calc_Distance(double latitude1, double longitude1, double latitude2, double longitude2);
  double res, lat1, long1, lat2, long2;
  lat1 = 48.89191;
  long1 = 35.29989;
  lat2 =48.89;
  long1 = 35.298;

  //res = Calc_Distance(lat1, long1, lat2, long1);
  res = haversine_km(lat1, long1, lat2, long1);
  if (res > 300)
  {
    lat2 = 0;
  }
  while (res < 10);

  //char test[] = "$GPRMC,001252.000,A,4831.6421,N,03501.9975,E,60.3,139.4,280611,,,A*60";
  //GPS_GPRMC result = NMEA_Parse_GPRMC(test);

  //600 =183
  //435 = 110
  // 385 = 100
  //350 = 80
  // 300 = 60
  //280 = 50
  // 250 = 40
  //220 = 30
  //190 = 20
  // 150 = 10
  //calc_pwm_speed(15);

 init_hardware();

 __enable_interrupt();



 GPS_GPRMC result;
 result.GPS_SPEED = 180;

 uint16_t pwm_value = calc_pwm_speed(result.GPS_SPEED);
 TIM2_Cmd(DISABLE);
 TIM2_OC2Init(TIM2_OCMODE_PWM2, TIM2_OUTPUTSTATE_ENABLE, pwm_value , TIM2_OCPOLARITY_LOW);
 TIM2_Cmd(ENABLE);

  while (sys_sec < 1);
 result.GPS_SPEED = 0;
 pwm_value = calc_pwm_speed(result.GPS_SPEED);
 TIM2_Cmd(DISABLE);
 TIM2_OC2Init(TIM2_OCMODE_PWM2, TIM2_OUTPUTSTATE_ENABLE, pwm_value , TIM2_OCPOLARITY_LOW);
 TIM2_Cmd(ENABLE);

  uint32_t sys_sec1 = 0;


  while(1)
  {
     if (UART_ACTUAL_LN <  UART_RX_BUF_LN_MAX)
     {
       //    
       uint8_t cmd_code = NMEA_Get_Cmd_Code((char *)UART_RX_BUF[UART_ACTUAL_LN]);
       if (cmd_code == NMEA_GPRMC)
       {
          UART1_Cmd(DISABLE);
          GPS_GPRMC result = NMEA_Parse_GPRMC((char *)UART_RX_BUF[UART_ACTUAL_LN]);

          if (result.VALID)
          {
            LED_BUF[0] = result.KSPEED[0];
            LED_BUF[1] = result.KSPEED[1];
            LED_BUF[2] = result.KSPEED[2];
            LED_BUF[3] = result.KSPEED[3];
            if (LED_BUF[3] == 0x10)
              LED_BUF[3] = 0;;
            LED_BUF_DP[1] = 0;
            uint16_t pwm_value = calc_pwm_speed(result.GPS_SPEED);
             TIM2_Cmd(DISABLE);
             TIM2_OC2Init(TIM2_OCMODE_PWM2, TIM2_OUTPUTSTATE_ENABLE, pwm_value , TIM2_OCPOLARITY_LOW);
             TIM2_Cmd(ENABLE);
          }
          else
          {
            LED_BUF[0] = result.CTIME[0];
            LED_BUF[1] = result.CTIME[1];
            LED_BUF[2] = result.CTIME[2];
            LED_BUF[3] = result.CTIME[3];
            if (sys_sec1 != sys_sec)
            {
              if (LED_BUF_DP[1])
                LED_BUF_DP[1] = 0;
              else
                LED_BUF_DP[1] = 1;
              sys_sec1 = sys_sec;
            }
          }
          UART1_Cmd(ENABLE);
       }
       UART_ACTUAL_LN = UART_RX_BUF_LN_MAX + 1;
     }
  }

}
